@@ -74,8 +74,8 @@ typedef struct _gswindow_device_t {
   int                   screen;        /* Screeen this window is on */
   GC                    gc;            /* GC for drawing */
   long                  number;        /* Globally unique identifier */
-  int                   depth;         /* Window depth */
-  int                   border;        /* Border size */
+  unsigned int          depth;         /* Window depth */
+  unsigned int          border;        /* Border size */
   int			map_state;     /* X map state */
   int                   visibility;    /* X visibility */
   NSBackingStoreType    type;          /* Backing type */
@@ -119,6 +119,6 @@ typedef struct _gswindow_device_t {
 @end
 
 extern Pixmap
-xgps_cursor_mask(Display *xdpy, Drawable draw, const char *data, 
+xgps_cursor_mask(Display *xdpy, Drawable draw, const unsigned char *data,
 		 int w, int h, int colors);
 #endif
