@@ -34,6 +34,8 @@
 	color: (unsigned char)r : (unsigned char)g : (unsigned char)b
 	: (unsigned char)alpha
 	transform: (NSAffineTransform *)transform
+	deltas: (const float *)delta_data : (int)delta_size : (int)delta_flags
+        widthChar: (int) wch
 	drawinfo: (struct draw_info_s *)di;
 
 -(void) drawGlyphs: (const NSGlyph *)glyphs : (int)length
@@ -55,14 +57,6 @@
 	transform: (NSAffineTransform *)transform
 	drawinfo: (struct draw_info_s *)di;
 
-/* TODO: see if this is really necessary */
--(void) drawString: (const char *)s
-	at: (int)x:(int)y
-	to: (int)x0:(int)y0:(int)x1:(int)y1:(unsigned char *)buf:(int)bpl
-	color:(unsigned char)r:(unsigned char)g:(unsigned char)b:(unsigned char)alpha
-	transform: (NSAffineTransform *)transform
-	deltas: (const float *)delta_data : (int)delta_size : (int)delta_flags;
-
 -(void) outlineString: (const char *)s
 	at: (float)x : (float)y
 	gstate: (void *)func_param;
