@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
    Author:  Alexander Malmberg <alexander@malmberg.org>
 
@@ -20,6 +20,18 @@
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+
+#include <ft2build.h>
+#include FT_FREETYPE_H
+
+
+#if (FREETYPE_MAJOR==2) && ((FREETYPE_MINOR<1) || ((FREETYPE_MINOR==1) && (FREETYPE_PATCH<8)))
+
+#include "ftfont-old.m"
+
+#else
+
+
 #include <math.h>
 
 #include <Foundation/NSObject.h>
@@ -49,8 +61,6 @@
 
 /** font handling interface **/
 
-#include <ft2build.h>
-#include FT_FREETYPE_H
 #include FT_CACHE_H
 
 #include FT_CACHE_IMAGE_H
@@ -59,10 +69,6 @@
 
 #include FT_OUTLINE_H
 
-#if (FREETYPE_MAJOR==2) && ((FREETYPE_MINOR<1) || ((FREETYPE_MINOR==1) && (FREETYPE_PATCH<=2)))
-#define FT212_STUFF
-#endif
-
 
 /* TODO: finish screen font handling */
 
@@ -86,21 +92,18 @@ static BOOL anti_alias_by_default;
 @interface FTFontInfo : GSFontInfo <FTFontInfo>
 {
 @public
-#ifdef FT212_STUFF
-  FTC_ImageDesc imgd;
+  int pix_width, pix_height;
 
-  FTC_ImageDesc advancementImgd;
-#else
-  FTC_ImageTypeRec imgd;
-
-  FTC_ImageTypeRec advancementImgd;
-#endif
-
-  FTFaceInfo *face_info;
+  FTC_FaceID faceId;
+  FTC_ImageTypeRec imageType;
+  FTC_ScalerRec scaler;
+  int unicodeCmap;
 
   BOOL screenFont;
 
 
+  FTFaceInfo *face_info;
+
   /*
   Profiling (2003-11-14) shows that calls to -advancementForGlyph: accounted
   for roughly 20% of layout time. This cache reduces it to (currently)
@@ -296,7 +299,10 @@ static NSArray *fix_path(NSString *path, NSArray *files)
   nfiles = [[NSMutableArray alloc] init];
   for (i = 0; i < c; i++)
     {
-      [nfiles addObject: [path stringByAppendingPathComponent:
+      if ([[files objectAtIndex: i] isAbsolutePath])
+	[nfiles addObject: [files objectAtIndex: i]];
+      else
+	[nfiles addObject: [path stringByAppendingPathComponent:
 	  [files objectAtIndex: i]]];
     }
   return nfiles;
@@ -308,7 +314,7 @@ static void add_face(NSString *family, int family_weight,
 	BOOL from_nfont)
 {
   FTFaceInfo *fi;
-  int weight;
+  unsigned int weight;
   unsigned int traits;
 
   NSString *fontName;
@@ -364,7 +370,7 @@ static void add_face(NSString *family, int family_weight,
     }
   else if (!from_nfont)
     { /* try to guess something for .font packages */
-      int dummy;
+      unsigned int dummy;
       int split = traits_from_string(family,&dummy,&dummy);
       rawFaceName = faceName = [family substringFromIndex: split];
       family = [family substringToIndex: split];
@@ -423,13 +429,14 @@ static void add_face(NSString *family, int family_weight,
   fi->traits = traits;
 
   if ([d objectForKey: @"RenderHints_hack"])
-    fi->render_hints_hack=strtol([[d objectForKey: @"RenderHints_hack"] cString],NULL,0);
+    fi->render_hints_hack
+      = strtol([[d objectForKey: @"RenderHints_hack"] cString], NULL, 0);
   else
     {
       if (anti_alias_by_default)
-	fi->render_hints_hack=0x10202;
+	fi->render_hints_hack = 0x10202;
       else
-	fi->render_hints_hack=0x00202;
+	fi->render_hints_hack = 0x00202;
     }
 
   NSDebugLLog(@"ftfont", @"adding '%@' '%@'", fontName, fi);
@@ -667,7 +674,6 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	matrix: (const float *)fmatrix
 	screenFont: (BOOL)p_screenFont
 {
-  FT_Face face;
   FT_Size size;
   NSArray *rfi;
   FTFaceInfo *font_entry;
@@ -707,13 +713,14 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
   familyName = [face_info->familyName copy];
   memcpy(matrix, fmatrix, sizeof(matrix));
 
-  /* TODO: somehow make gnustep-gui send unicode our way. utf8? ugly, but it works */
+  /* Using utf8 is a bit ugly, but it works.  Besides, the bulk of the text
+     comes as glyphs anyway.  */
   mostCompatibleStringEncoding = NSUTF8StringEncoding;
   encodingScheme = @"iso10646-1";
 
   if (screenFont)
     {
-      /* Round up; makes the text more legible. */
+      /* Round up; makes the text more legible.  */
       matrix[0] = ceil(matrix[0]);
       if (matrix[3] < 0.0)
 	matrix[3] = floor(matrix[3]);
@@ -721,17 +728,17 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	matrix[3] = ceil(matrix[3]);
     }
 
-  imgd.font.pix_width = fabs(matrix[0]);
-  imgd.font.pix_height = fabs(matrix[3]);
+  pix_width = fabs(matrix[0]);
+  pix_height = fabs(matrix[3]);
 
   rfi = font_entry->files;
   if (screenFont && font_entry->num_sizes &&
-      imgd.font.pix_width == imgd.font.pix_height)
+      pix_width == pix_height)
     {
       int i;
       for (i = 0; i < font_entry->num_sizes; i++)
 	{
-	  if (font_entry->sizes[i].pixel_size == imgd.font.pix_width)
+	  if (font_entry->sizes[i].pixel_size == pix_width)
 	    {
 	      rfi = font_entry->sizes[i].files;
 	      break;
@@ -739,11 +746,21 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	}
     }
 
-  imgd.font.face_id = (FTC_FaceID)rfi;
+  faceId = (FTC_FaceID)rfi;
 
-  if ((error=FTC_Manager_Lookup_Size(ftc_manager, &imgd.font, &face, &size)))
+  imageType.face_id = faceId;
+  imageType.width = pix_width;
+  imageType.height = pix_height;
+  /* TODO: Flags? */
+
+  scaler.face_id = faceId;
+  scaler.width = pix_width;
+  scaler.height = pix_height;
+  scaler.pixel = 1;
+  
+  if ((error = FTC_Manager_LookupSize(ftc_manager, &scaler, &size)))
     {
-      NSLog(@"FTC_Manager_Lookup_Size() failed for '%@', error %08x!\n", name, error);
+      NSLog(@"FTC_Manager_LookupSize() failed for '%@', error %08x!\n", name, error);
       return self;
     }
 
@@ -766,15 +783,28 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 		fontBBox.size.width, fontBBox.size.height);*/
 
   {
-    FTC_CMapDescRec cmap;
-    cmap.face_id = imgd.font.face_id;
-    cmap.u.encoding = ft_encoding_unicode;
-    cmap.type = FTC_CMAP_BY_ENCODING;
-    ligature_ff = FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, 0xfb00);
-    ligature_fi = FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, 0xfb01);
-    ligature_fl = FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, 0xfb02);
-    ligature_ffi = FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, 0xfb03);
-    ligature_ffl = FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, 0xfb04);
+    int i;
+    FT_Face face = size->face;
+    FT_CharMap cmap;
+    FT_Encoding e;
+    unicodeCmap = 0;
+    for (i = 0; i < face->num_charmaps; i++)
+      {
+	cmap = face->charmaps[i];
+	e = cmap->encoding;
+	if (e == FT_ENCODING_UNICODE)
+	  {
+	    unicodeCmap = i;
+	    break;
+	  }
+      }
+  }
+  {
+    ligature_ff = FTC_CMapCache_Lookup(ftc_cmapcache, faceId, unicodeCmap, 0xfb00);
+    ligature_fi = FTC_CMapCache_Lookup(ftc_cmapcache, faceId, unicodeCmap,  0xfb01);
+    ligature_fl = FTC_CMapCache_Lookup(ftc_cmapcache, faceId, unicodeCmap, 0xfb02);
+    ligature_ffi = FTC_CMapCache_Lookup(ftc_cmapcache, faceId, unicodeCmap, 0xfb03);
+    ligature_ffl = FTC_CMapCache_Lookup(ftc_cmapcache, faceId, unicodeCmap, 0xfb04);
 /*    printf("ligatures %04x %04x %04x %04x %04x | %02x %02x %02x for |%@|\n",
       ligature_ff,ligature_fi,ligature_fl,ligature_ffi,ligature_ffl,
       FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, 'f'),
@@ -783,67 +813,46 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
       fontName);*/
   }
 
+  if (screenFont)
     {
-      float xx, yy;
-#ifdef FT212_STUFF
-      FTC_ImageDesc cur;
-#else
-      FTC_ImageTypeRec cur;
-#endif
-
-      cur = imgd;
-
-      xx = matrix[0];
-      yy = matrix[3];
-
-	if (xx == yy && xx < 16 && xx >= 8)
-	  {
-	    int rh = face_info->render_hints_hack;
-	    if (rh & 0x10000)
-	      {
-#ifdef FT212_STUFF
-		cur.type = ftc_image_grays;
-#else
-		cur.flags = FT_LOAD_TARGET_NORMAL;
-#endif
-		rh = (rh >> 8) & 0xff;
-	      }
-	    else
-	      {
-#ifdef FT212_STUFF
-		cur.type = ftc_image_mono;
-#else
-		cur.flags = FT_LOAD_TARGET_MONO;
-#endif
-		rh = rh & 0xff;
-	      }
-	    if (rh & 1)
-#ifdef FT212_STUFF
-	      cur.type |= ftc_image_flag_autohinted;
-#else
-	      cur.flags |= FT_LOAD_FORCE_AUTOHINT;
-#endif
-	    if (!(rh & 2))
-#ifdef FT212_STUFF
-	      cur.type |= ftc_image_flag_unhinted;
-#else
-	      cur.flags |= FT_LOAD_NO_HINTING;
-#endif
-	  }
-	else if (xx < 8)
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays | ftc_image_flag_unhinted;
-#else
-	  cur.flags = FT_LOAD_TARGET_NORMAL | FT_LOAD_NO_HINTING;
-#endif
-	else
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays;
-#else
-	  cur.flags = FT_LOAD_TARGET_NORMAL;
-#endif
-      advancementImgd = cur;
+      int flags;
+      if (pix_width == pix_height && pix_width < 16 && pix_height >= 8)
+	{
+	  int rh = face_info->render_hints_hack;
+	  if (rh & 0x10000)
+	    {
+	      flags = FT_LOAD_TARGET_NORMAL;
+	      rh = (rh >> 8) & 0xff;
+	    }
+	  else
+	    {
+	      flags = FT_LOAD_TARGET_MONO;
+	      rh = rh & 0xff;
+	    }
+	  if (rh & 1)
+	    flags |= FT_LOAD_FORCE_AUTOHINT;
+	  if (!(rh & 2))
+	    flags |= FT_LOAD_NO_HINTING;
+	}
+      else if (pix_width < 8)
+	flags = FT_LOAD_TARGET_NORMAL | FT_LOAD_NO_HINTING;
+      else
+	flags = FT_LOAD_TARGET_NORMAL;
+      imageType.flags = flags;
     }
+  else
+    {
+      imageType.flags = FT_LOAD_NO_HINTING;
+    }
+
+  /*
+  Here, we simply need to make sure that we don't get any false matches
+  the first time a particular cache entry is used. Thus, we only need to
+  initialize the first entry. For all other entries, cachedGlyph[i] will
+  be 0, and that's a glyph that can't possibly hash to any entry except
+  entry #0, so it won't cause any false matches.
+  */
+  cachedGlyph[0] = 1;
 
   return self;
 }
@@ -864,17 +873,18 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 
 /* TODO: the current point probably needs updating after drawing is done */
 
--(void) drawString: (const char *)s
-	at: (int)x:(int)y
-	to: (int)x0:(int)y0:(int)x1:(int)y1:(unsigned char *)buf:(int)bpl
-	color:(unsigned char)r:(unsigned char)g:(unsigned char)b:(unsigned char)alpha
-	transform: (NSAffineTransform *)transform
-	deltas: (const float *)delta_data : (int)delta_size : (int)delta_flags;
-{
-/* TODO */
-}
-
-
+/* draw string at point, clipped, w/given color and alpha, and possible deltas:
+   flags & 0x1: data contains x offsets, use instead of glyph x advance
+   flags & 0x2: data contains y offsets, use instead of glyph y advance
+   flags & 0x4: data contains a single x and y offset, which should be added to
+                font's advancements for each glyph; results are undefined if
+                this option is combined with either x or y offsets (0x1,0x2)
+   flags & 0x8: data contains a single x and y offset, which should be added to
+                font's advancement for glyph identified by 'wch'; if combined
+                with 0x4 deltas contain exactly two offsets for x and y, the
+                first for every character, the second for 'wch'; results are
+                undefined if 0x8 is combined with 0x2 or 0x1
+ */
 -(void) drawString: (const char *)s
 	at: (int)x : (int)y
 	to: (int)x0 : (int)y0 : (int)x1 : (int)y1
@@ -882,26 +892,20 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	: (unsigned char *)abuf : (int)abpl
 	color:(unsigned char)r : (unsigned char)g : (unsigned char)b : (unsigned char)alpha
 	transform: (NSAffineTransform *)transform
+	deltas: (const float *)delta_data : (int)delta_size : (int)delta_flags
+        widthChar: (int) wch
 	drawinfo: (draw_info_t *)di
 {
-#if 0
-  NSLog(@"ignoring drawString");
-#else
   const unsigned char *c;
   unsigned char ch;
   unsigned int uch;
+  int d;
 
-  FTC_CMapDescRec cmap;
   unsigned int glyph;
 
   int use_sbit;
 
   FTC_SBit sbit;
-#ifdef FT212_STUFF
-  FTC_ImageDesc cur;
-#else
-  FTC_ImageTypeRec cur;
-#endif
 
   FT_Matrix ftmatrix;
   FT_Vector ftdelta;
@@ -928,7 +932,6 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 		transform
 		);*/
 
-  cur = imgd;
   {
     float xx, xy, yx, yy;
 
@@ -945,70 +948,31 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	xx < 72 && yy < 72 && xx > 0.5 && yy > 0.5)
       {
 	use_sbit = 1;
-	cur.font.pix_width = xx;
-	cur.font.pix_height = yy;
-
-	if (xx == yy && xx < 16 && xx >= 8)
-	  {
-	    int rh = face_info->render_hints_hack;
-	    if (rh & 0x10000)
-	      {
-#ifdef FT212_STUFF
-		cur.type = ftc_image_grays;
-#else
-		cur.flags = FT_LOAD_TARGET_NORMAL;
-#endif
-		rh = (rh >> 8) & 0xff;
-	      }
-	    else
-	      {
-#ifdef FT212_STUFF
-		cur.type = ftc_image_mono;
-#else
-		cur.flags = FT_LOAD_TARGET_MONO;
-#endif
-		rh = rh & 0xff;
-	      }
-	    if (rh & 1)
-#ifdef FT212_STUFF
-	      cur.type |= ftc_image_flag_autohinted;
-#else
-	      cur.flags |= FT_LOAD_FORCE_AUTOHINT;
-#endif
-	    if (!(rh & 2))
-#ifdef FT212_STUFF
-	      cur.type |= ftc_image_flag_unhinted;
-#else
-	      cur.flags |= FT_LOAD_NO_HINTING;
-#endif
-	  }
-	else if (xx < 8)
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays | ftc_image_flag_unhinted;
-#else
-	  cur.flags = FT_LOAD_TARGET_NORMAL | FT_LOAD_NO_HINTING;
-#endif
-	else
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays;
-#else
-	  cur.flags = FT_LOAD_TARGET_NORMAL;
-#endif
       }
     else
       {
 	float f;
 	use_sbit = 0;
 
-	f = fabs(xx * yy - xy * yx);
+	/* TODO: Think hard about this.  Fancy truetype magic can
+	   significantly change the outline, e.g. by adding fancy adornments
+	   at large sizes.  If the font is then scaled down, we should
+	   probably set the font size to the original size and scale down
+	   using the matrix.  This ensures that adornments and such are
+	   still present, just scaled down.
+
+	   This is somewhat complicated by the fact that we don't have any
+	   real size, just a matrix.  Thus, we average pix_width and
+	   pix_height; we'll get the right answer for normal cases, and
+	   we can't really do anything about the weird cases.  */
+	f = fabs(pix_width) + fabs(pix_height);
 	if (f > 1)
-	  f = sqrt(f);
+	  f = f / 2.0;
 	else
 	  f = 1.0;
-
 	f = (int)f;
 
-	cur.font.pix_width = cur.font.pix_height = f;
+	scaler.width = scaler.height = f;
 	ftmatrix.xx = xx / f * 65536.0;
 	ftmatrix.xy = xy / f * 65536.0;
 	ftmatrix.yx = yx / f * 65536.0;
@@ -1021,11 +985,7 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 /*	NSLog(@"drawString: '%s' at: %i:%i  to: %i:%i:%i:%i:%p\n",
 		s, x, y, x0, y0, x1, y1, buf);*/
 
-  cmap.face_id = imgd.font.face_id;
-  cmap.u.encoding = ft_encoding_unicode;
-  cmap.type = FTC_CMAP_BY_ENCODING;
-
-  for (c = s; *c; c++)
+  for (c = (const unsigned char *)s; *c; c++)
     {
 /* TODO: do the same thing in outlineString:... */
       ch = *c;
@@ -1079,27 +1039,58 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	uch = 0xfffd;
 #undef ADD_UTF_BYTE
 
-      glyph = FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, uch);
-      cur.font.face_id = imgd.font.face_id;
+      glyph = FTC_CMapCache_Lookup(ftc_cmapcache, faceId, unicodeCmap, uch);
 
       if (use_sbit)
 	{
-	  if ((error=FTC_SBitCache_Lookup(ftc_sbitcache, &cur, glyph, &sbit, NULL)))
+	  if ((error = FTC_SBitCache_Lookup(ftc_sbitcache, &imageType, glyph, &sbit, NULL)))
 	    {
 	      NSLog(@"FTC_SBitCache_Lookup() failed with error %08x (%08x, %08x, %ix%i, %08x)\n",
-		error, glyph, cur.font.face_id, cur.font.pix_width, cur.font.pix_height,
-#ifdef FT212_STUFF
-		cur.type
-#else
-		cur.flags
-#endif
-		);
+		error, glyph, imageType.face_id, imageType.width, imageType.height,
+		imageType.flags);
 	      continue;
 	    }
 
 	  if (!sbit->buffer)
 	    {
-	      x += sbit->xadvance;
+              if (!delta_flags)
+                {
+                x += sbit->xadvance;
+                }
+              else
+                {
+                  if (delta_flags & 0x1)
+                    x += delta_data[d++];
+                  if (delta_flags & 0x2)
+                    y += (transform->matrix.m22 < 0) ?
+                        delta_data[d++] : -delta_data[d++];
+                  if (delta_flags & 0x4)
+                    {
+                      x += sbit->xadvance + delta_data[0];
+                      y += /*sbit->yadvance +*/ (transform->matrix.m22 < 0) ?
+                          delta_data[1] : -delta_data[1];
+                      if ((delta_flags & 0x8) && (uch == wch))
+                        {
+                          x += delta_data[2];
+                          y += (transform->matrix.m22 < 0) ?
+                              delta_data[3] : -delta_data[3];
+                        }
+                    }
+                  else if (delta_flags & 0x8)
+                    {
+                      if (uch == wch)
+                        {
+                          x += sbit->xadvance + delta_data[0];
+                          y += /*sbit->yadvance +*/ (transform->matrix.m22 < 0) ?
+                            delta_data[1] : -delta_data[1];
+                        }
+                      else
+                        {
+                          x += sbit->xadvance;
+                          /*y += sbit->yadvance;*/
+                        }
+                    }
+                }
 	      continue;
 	    }
 
@@ -1208,22 +1199,60 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	      NSLog(@"unhandled font bitmap format %i", sbit->format);
 	    }
 
-	  x += sbit->xadvance;
+          if (!delta_flags)
+            {
+              x += sbit->xadvance;
+            }
+          else
+            {
+              if (delta_flags & 0x1)
+                  x += delta_data[d++];
+              if (delta_flags & 0x2)
+                  y += (transform->matrix.m22 < 0) ?
+                        delta_data[d++] : -delta_data[d++];
+              if (delta_flags & 0x4)
+                {
+                  x += sbit->xadvance + delta_data[0];
+                  y += /*sbit->yadvance +*/ (transform->matrix.m22 < 0) ?
+                          delta_data[1] : -delta_data[1];
+                  if ((delta_flags & 0x8) && (uch == wch))
+                    {
+                      x += delta_data[2];
+                      y += (transform->matrix.m22 < 0) ?
+                              delta_data[3] : -delta_data[3];
+                    }
+                }
+              else if (delta_flags & 0x8)
+                {
+                  if (uch == wch)
+                    {
+                      x += sbit->xadvance + delta_data[0];
+                      y += /*sbit->yadvance +*/ (transform->matrix.m22 < 0) ?
+                          delta_data[1] : -delta_data[1];
+                    }
+                  else
+                    {
+                      x += sbit->xadvance;
+                      /*y += sbit->yadvance;*/
+                    }
+                }
+            }
 	}
       else
 	{
 	  FT_Face face;
 	  FT_Glyph gl;
+	  FT_Size size;
 	  FT_BitmapGlyph gb;
 
-	  if ((error=FTC_Manager_Lookup_Size(ftc_manager, &cur.font, &face, 0)))
+	  if ((error=FTC_Manager_LookupSize(ftc_manager, &scaler, &size)))
 	    {
 	      NSLog(@"FTC_Manager_Lookup_Size() failed with error %08x\n",error);
 	      continue;
 	    }
+	  face = size->face;
 
-	  /* TODO: for rotations of 90, 180, 270, and integer
-	     scales hinting might still be a good idea. */
+	  /* TODO: for screen fonts we should probably still hint */
 	  if ((error=FT_Load_Glyph(face, glyph, FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP)))
 	    {
 	      NSLog(@"FT_Load_Glyph() failed with error %08x\n",error);
@@ -1314,14 +1343,45 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	      NSLog(@"unhandled font bitmap format %i", gb->bitmap.pixel_mode);
 	    }
 
-	  ftdelta.x += gl->advance.x >> 10;
-	  ftdelta.y += gl->advance.y >> 10;
+          if (!delta_flags)
+            {
+              ftdelta.x += gl->advance.x >> 10;
+              ftdelta.y += gl->advance.y >> 10;
+            }
+          else
+            {
+              if (delta_flags & 0x1)
+                ftdelta.x += delta_data[d++] * 64.0;
+              if (delta_flags & 0x2)
+                ftdelta.y += delta_data[d++] * 64.0;
+              if (delta_flags & 0x4)
+                {
+                  ftdelta.x += (gl->advance.x >> 10) + delta_data[0] * 64.0;
+                  ftdelta.y += (gl->advance.y >> 10) + delta_data[1] * 64.0;
+                  if ((delta_flags & 0x8) && (uch == wch))
+                    {
+                      ftdelta.x += delta_data[2] * 64.0;
+                      ftdelta.y += delta_data[3] * 64.0;
+                    }
+                }
+              else if (delta_flags & 0x8)
+                {
+                  if (uch == wch)
+                    {
+                      ftdelta.x += (gl->advance.x >> 10) + delta_data[0] * 64.0;
+                      ftdelta.y += (gl->advance.y >> 10) + delta_data[1] * 64.0;
+                    }
+                  else
+                    {
+                      ftdelta.x += gl->advance.x >> 10;
+                      ftdelta.y += gl->advance.y >> 10;
+                    }
+                }
+            }
 
 	  FT_Done_Glyph(gl);
 	}
     }
-
-#endif
 }
 
 
@@ -1339,11 +1399,6 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
   int use_sbit;
 
   FTC_SBit sbit;
-#ifdef FT212_STUFF
-  FTC_ImageDesc cur;
-#else
-  FTC_ImageTypeRec cur;
-#endif
 
   FT_Matrix ftmatrix;
   FT_Vector ftdelta;
@@ -1370,7 +1425,6 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 		transform
 		);*/
 
-  cur = imgd;
   {
     float xx, xy, yx, yy;
 
@@ -1387,70 +1441,20 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	xx < 72 && yy < 72 && xx > 0.5 && yy > 0.5)
       {
 	use_sbit = 1;
-	cur.font.pix_width = xx;
-	cur.font.pix_height = yy;
-
-	if (xx == yy && xx < 16 && xx >= 8)
-	  {
-	    int rh = face_info->render_hints_hack;
-	    if (rh & 0x10000)
-	      {
-#ifdef FT212_STUFF
-		cur.type = ftc_image_grays;
-#else
-		cur.flags = FT_LOAD_TARGET_NORMAL;
-#endif
-		rh = (rh >> 8) & 0xff;
-	      }
-	    else
-	      {
-#ifdef FT212_STUFF
-		cur.type = ftc_image_mono;
-#else
-		cur.flags = FT_LOAD_TARGET_MONO;
-#endif
-		rh = rh & 0xff;
-	      }
-	    if (rh & 1)
-#ifdef FT212_STUFF
-	      cur.type |= ftc_image_flag_autohinted;
-#else
-	      cur.flags |= FT_LOAD_FORCE_AUTOHINT;
-#endif
-	    if (!(rh & 2))
-#ifdef FT212_STUFF
-	      cur.type |= ftc_image_flag_unhinted;
-#else
-	      cur.flags |= FT_LOAD_NO_HINTING;
-#endif
-	  }
-	else if (xx < 8)
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays | ftc_image_flag_unhinted;
-#else
-	  cur.flags = FT_LOAD_TARGET_NORMAL | FT_LOAD_NO_HINTING;
-#endif
-	else
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays;
-#else
-	  cur.flags = FT_LOAD_TARGET_NORMAL;
-#endif
       }
     else
       {
 	float f;
 	use_sbit = 0;
 
-	f = fabs(xx * yy - xy * yx);
+	f = fabs(pix_width) + fabs(pix_height);
 	if (f > 1)
-	  f = sqrt(f);
+	  f = f / 2.0;
 	else
 	  f = 1.0;
-
 	f = (int)f;
 
-	cur.font.pix_width = cur.font.pix_height = f;
+	scaler.width = scaler.height = f;
 	ftmatrix.xx = xx / f * 65536.0;
 	ftmatrix.xy = xy / f * 65536.0;
 	ftmatrix.yx = yx / f * 65536.0;
@@ -1459,8 +1463,8 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
       }
   }
 
-/*	NSLog(@"drawString: '%s' at: %i:%i  to: %i:%i:%i:%i:%p\n",
-		s, x, y, x0, y0, x1, y1, buf);*/
+/*	NSLog(@"drawGlyphs: '%p' at: %i:%i  to: %i:%i:%i:%i:%p\n",
+		glyphs, x, y, x0, y0, x1, y1, buf);*/
 
   for (; length; length--, glyphs++)
     {
@@ -1468,15 +1472,11 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 
       if (use_sbit)
 	{
-	  if ((error = FTC_SBitCache_Lookup(ftc_sbitcache, &cur, glyph, &sbit, NULL)))
+	  if ((error = FTC_SBitCache_Lookup(ftc_sbitcache, &imageType, glyph, &sbit, NULL)))
 	    {
 	      NSLog(@"FTC_SBitCache_Lookup() failed with error %08x (%08x, %08x, %ix%i, %08x)\n",
-		error, glyph, cur.font.face_id, cur.font.pix_width, cur.font.pix_height,
-#ifdef FT212_STUFF
-		cur.type
-#else
-		cur.flags
-#endif
+		error, glyph, imageType.face_id, imageType.width, imageType.height,
+		imageType.flags
 		);
 	      continue;
 	    }
@@ -1597,16 +1597,18 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
       else
 	{
 	  FT_Face face;
+	  FT_Size size;
 	  FT_Glyph gl;
 	  FT_BitmapGlyph gb;
 
-	  if ((error=FTC_Manager_Lookup_Size(ftc_manager, &cur.font, &face, 0)))
+	  if ((error=FTC_Manager_LookupSize(ftc_manager, &scaler, &size)))
 	    {
 	      NSLog(@"FTC_Manager_Lookup_Size() failed with error %08x\n",error);
 	      continue;
 	    }
+	  face = size->face;
 
-	  /* TODO: for rotations of 90, 180, 270, and integer
+	  /* TODO: for screen fonts ...see above... rotations of 90, 180, 270, and integer
 	     scales hinting might still be a good idea. */
 	  if ((error=FT_Load_Glyph(face, glyph, FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP)))
 	    {
@@ -1713,11 +1715,6 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
   int use_sbit;
 
   FTC_SBit sbit;
-#ifdef FT212_STUFF
-  FTC_ImageDesc cur;
-#else
-  FTC_ImageTypeRec cur;
-#endif
 
   FT_Matrix ftmatrix;
   FT_Vector ftdelta;
@@ -1744,7 +1741,6 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 		transform
 		);*/
 
-  cur = imgd;
   {
     float xx, xy, yx, yy;
 
@@ -1761,70 +1757,20 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 	xx < 72 && yy < 72 && xx > 0.5 && yy > 0.5)
       {
 	use_sbit = 1;
-	cur.font.pix_width = xx;
-	cur.font.pix_height = yy;
-
-	if (xx == yy && xx < 16 && xx >= 8)
-	  {
-	    int rh = face_info->render_hints_hack;
-	    if (rh & 0x10000)
-	      {
-#ifdef FT212_STUFF
-		cur.type = ftc_image_grays;
-#else
-		cur.flags = FT_LOAD_TARGET_NORMAL;
-#endif
-		rh = (rh >> 8) & 0xff;
-	      }
-	    else
-	      {
-#ifdef FT212_STUFF
-		cur.type = ftc_image_mono;
-#else
-		cur.flags = FT_LOAD_TARGET_MONO;
-#endif
-		rh = rh & 0xff;
-	      }
-	    if (rh & 1)
-#ifdef FT212_STUFF
-	      cur.type |= ftc_image_flag_autohinted;
-#else
-	      cur.flags |= FT_LOAD_FORCE_AUTOHINT;
-#endif
-	    if (!(rh & 2))
-#ifdef FT212_STUFF
-	      cur.type |= ftc_image_flag_unhinted;
-#else
-	      cur.flags |= FT_LOAD_NO_HINTING;
-#endif
-	  }
-	else if (xx < 8)
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays | ftc_image_flag_unhinted;
-#else
-	  cur.flags = FT_LOAD_TARGET_NORMAL | FT_LOAD_NO_HINTING;
-#endif
-	else
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays;
-#else
-	  cur.flags = FT_LOAD_TARGET_NORMAL;
-#endif
       }
     else
       {
 	float f;
 	use_sbit = 0;
 
-	f = fabs(xx * yy - xy * yx);
+	f = fabs(pix_width) + fabs(pix_height);
 	if (f > 1)
-	  f = sqrt(f);
+	  f = f / 2.0;
 	else
 	  f = 1.0;
-
 	f = (int)f;
 
-	cur.font.pix_width = cur.font.pix_height = f;
+	scaler.width = scaler.height = f;
 	ftmatrix.xx = xx / f * 65536.0;
 	ftmatrix.xy = xy / f * 65536.0;
 	ftmatrix.yx = yx / f * 65536.0;
@@ -1842,16 +1788,12 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 
       if (use_sbit)
 	{
-	  if ((error = FTC_SBitCache_Lookup(ftc_sbitcache, &cur, glyph, &sbit, NULL)))
+	  if ((error = FTC_SBitCache_Lookup(ftc_sbitcache, &imageType, glyph, &sbit, NULL)))
 	    {
 	      if (glyph != 0xffffffff)
 		NSLog(@"FTC_SBitCache_Lookup() failed with error %08x (%08x, %08x, %ix%i, %08x)\n",
-		  error, glyph, cur.font.face_id, cur.font.pix_width, cur.font.pix_height,
-#ifdef FT212_STUFF
-		  cur.type
-#else
-		  cur.flags
-#endif
+		  error, glyph, imageType.face_id, imageType.width, imageType.height,
+		  imageType.flags
 		  );
 	      continue;
 	    }
@@ -1970,16 +1912,18 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
       else
 	{
 	  FT_Face face;
+	  FT_Size size;
 	  FT_Glyph gl;
 	  FT_BitmapGlyph gb;
 
-	  if ((error=FTC_Manager_Lookup_Size(ftc_manager, &cur.font, &face, 0)))
+	  if ((error=FTC_Manager_LookupSize(ftc_manager, &scaler, &size)))
 	    {
 	      NSLog(@"FTC_Manager_Lookup_Size() failed with error %08x\n",error);
 	      continue;
 	    }
+	  face = size->face;
 
-	  /* TODO: for rotations of 90, 180, 270, and integer
+	  /* TODO: for screen fonts, see above, etc., rotations of 90, 180, 270, and integer
 	     scales hinting might still be a good idea. */
 	  if ((error=FT_Load_Glyph(face, glyph, FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP)))
 	    {
@@ -2074,14 +2018,16 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 -(BOOL) glyphIsEncoded: (NSGlyph)glyph
 {
   FT_Face face;
+  FT_Size size;
   FT_Error error;
 
   glyph--;
-  if ((error=FTC_Manager_Lookup_Size(ftc_manager, &imgd.font, &face, 0)))
+  if ((error=FTC_Manager_LookupSize(ftc_manager, &scaler, &size)))
     {
       NSLog(@"FTC_Manager_Lookup_Size() failed with error %08x",error);
       return NO;
     }
+  face = size->face;
 
   if ((error=FT_Load_Glyph(face, glyph, 0)))
     {
@@ -2098,11 +2044,11 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
   FT_Error error;
 
   if (glyph == NSControlGlyph
-   || glyph == GSAttachmentGlyph
-   || glyph == NSNullGlyph)
+   || glyph == GSAttachmentGlyph)
     return NSZeroSize;
 
-  glyph--;
+  if (glyph != NSNullGlyph)
+    glyph--;
   if (screenFont)
     {
       int entry = glyph % CACHE_SIZE;
@@ -2111,16 +2057,12 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
       if (cachedGlyph[entry] == glyph)
 	return cachedSize[entry];
 
-      if ((error=FTC_SBitCache_Lookup(ftc_sbitcache, &advancementImgd, glyph, &sbit, NULL)))
+      if ((error = FTC_SBitCache_Lookup(ftc_sbitcache, &imageType, glyph, &sbit, NULL)))
 	{
 	  NSLog(@"FTC_SBitCache_Lookup() failed with error %08x (%08x, %08x, %ix%i, %08x)\n",
-	    error, glyph, advancementImgd.font.face_id,
-	    advancementImgd.font.pix_width, advancementImgd.font.pix_height,
-#ifdef FT212_STUFF
-	    advancementImgd.type
-#else
-	    advancementImgd.flags
-#endif
+	    error, glyph, imageType.face_id,
+	    imageType.width, imageType.height,
+	    imageType.flags
 	    );
 	  return NSZeroSize;
 	}
@@ -2132,6 +2074,7 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
   else
     {
       FT_Face face;
+      FT_Size size;
       FT_Glyph gl;
       FT_Matrix ftmatrix;
       FT_Vector ftdelta;
@@ -2146,14 +2089,16 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 
       f = (int)f;
 
+      /* TODO? scalers and stuff, need to review */
       ftmatrix.xx = matrix[0] / f * 65536.0;
       ftmatrix.xy = matrix[1] / f * 65536.0;
       ftmatrix.yx = matrix[2] / f * 65536.0;
       ftmatrix.yy = matrix[3] / f * 65536.0;
       ftdelta.x = ftdelta.y = 0;
 
-      if (FTC_Manager_Lookup_Size(ftc_manager, &imgd.font, &face, 0))
+      if (FTC_Manager_LookupSize(ftc_manager, &scaler, &size))
 	return NSZeroSize;
+      face = size->face;
 
       if (FT_Load_Glyph(face, glyph, FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP))
 	return NSZeroSize;
@@ -2174,19 +2119,13 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 
 - (NSRect) boundingRectForGlyph: (NSGlyph)glyph
 {
-#ifdef FT212_STUFF
-  FTC_ImageDesc *cur;
-#else
-  FTC_ImageTypeRec *cur;
-#endif
   FT_BBox bbox;
   FT_Glyph g;
   FT_Error error;
 
   glyph--;
 /* TODO: this is ugly */
-  cur = &imgd;
-  if ((error=FTC_ImageCache_Lookup(ftc_imagecache, cur, glyph, &g, NULL)))
+  if ((error=FTC_ImageCache_Lookup(ftc_imagecache, &imageType, glyph, &g, NULL)))
     {
       NSLog(@"FTC_ImageCache_Lookup() failed with error %08x",error);
 //		NSLog(@"boundingRectForGlyph: %04x -> %i\n", aGlyph, glyph);
@@ -2208,6 +2147,7 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 {
   NSPoint a;
   FT_Face face;
+  FT_Size size;
   FT_Vector vec;
   FT_GlyphSlot glyph;
 
@@ -2220,8 +2160,9 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
   g--;
   prev--;
 
-  if (FTC_Manager_Lookup_Size(ftc_manager, &imgd.font, &face, 0))
+  if (FTC_Manager_LookupSize(ftc_manager, &scaler, &size))
     return NSZeroPoint;
+  face = size->face;
 
   if (FT_Load_Glyph(face, prev, FT_LOAD_DEFAULT))
     return NSZeroPoint;
@@ -2250,33 +2191,21 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
   int i, c = [string length];
   int total;
 
-  FTC_CMapDescRec cmap;
   unsigned int glyph;
 
   FTC_SBit sbit;
 
-#ifdef FT212_STUFF
-  FTC_ImageDesc *cur;
-#else
-  FTC_ImageTypeRec *cur;
-#endif
-
-
-  cmap.face_id = imgd.font.face_id;
-  cmap.u.encoding = ft_encoding_unicode;
-  cmap.type = FTC_CMAP_BY_ENCODING;
 
   total = 0;
   for (i = 0; i < c; i++)
     {
       ch = [string characterAtIndex: i];
-      cur = &imgd;
-      glyph = FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, ch);
+      glyph = FTC_CMapCache_Lookup(ftc_cmapcache, faceId, unicodeCmap, ch);
 
       /* TODO: shouldn't use sbit cache for this */
       if (1)
 	{
-	  if (FTC_SBitCache_Lookup(ftc_sbitcache, cur, glyph, &sbit, NULL))
+	  if (FTC_SBitCache_Lookup(ftc_sbitcache, &imageType, glyph, &sbit, NULL))
 	    continue;
 
 	  total += sbit->xadvance;
@@ -2293,10 +2222,12 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib, FT_Pointer data, FT_
 -(NSGlyph) glyphWithName: (NSString *)glyphName
 {
   FT_Face face;
+  FT_Size size;
   NSGlyph g;
 
-  if (FTC_Manager_Lookup_Size(ftc_manager, &imgd.font, &face, 0))
+  if (FTC_Manager_LookupSize(ftc_manager, &scaler, &size))
     return NSNullGlyph;
+  face = size->face;
 
   g = FT_Get_Name_Index(face, (FT_String *)[glyphName lossyCString]);
   if (g)
@@ -2534,6 +2465,7 @@ add code to avoid loading bitmaps for glyphs */
 		   at: (float)x : (float)y
 	       gstate: (void *)func_param
 {
+#if 0
   unichar *c;
   int i;
   FTC_CMapDescRec cmap;
@@ -2542,11 +2474,7 @@ add code to avoid loading bitmaps for glyphs */
   unichar *uch;
   int ulen;
 
-#ifdef FT212_STUFF
-  FTC_ImageDesc cur;
-#else
   FTC_ImageTypeRec cur;
-#endif
 
 
   FT_Matrix ftmatrix;
@@ -2610,6 +2538,7 @@ add code to avoid loading bitmaps for glyphs */
     }
 
   free(uch);
+#endif
 }
 
 
@@ -2617,6 +2546,7 @@ add code to avoid loading bitmaps for glyphs */
 			     count: (int)count
 		      toBezierPath: (NSBezierPath *)path
 {
+#if 0
   int i;
   NSGlyph glyph;
 
@@ -2667,6 +2597,7 @@ add code to avoid loading bitmaps for glyphs */
     {
       [path moveToPoint: NSMakePoint(ftdelta.x / 64.0, ftdelta.y / 64.0)];
     }
+#endif
 }
 
 
@@ -2758,6 +2689,7 @@ static int filters[3][7]=
 /* TODO: this whole thing needs cleaning up */
 @implementation FTFontInfo_subpixel
 
+#if 0
 -(void) drawGlyphs: (const NSGlyph *)glyphs : (int)length
 	at: (int)x : (int)y
 	to: (int)x0 : (int)y0 : (int)x1 : (int)y1
@@ -2773,11 +2705,7 @@ static int filters[3][7]=
   int use_sbit;
 
   FTC_SBit sbit;
-#ifdef FT212_STUFF
-  FTC_ImageDesc cur;
-#else
   FTC_ImageTypeRec cur;
-#endif
 
   FT_Matrix ftmatrix;
   FT_Vector ftdelta;
@@ -2825,11 +2753,7 @@ static int filters[3][7]=
 	    cur.font.pix_width > 6 && cur.font.pix_height > 6)
 	  cur.type = ftc_image_mono;
 	else*/
-#ifdef FT212_STUFF
-	  cur.type = ftc_image_grays, subpixel = YES, cur.font.pix_width *= 3, x *= 3;
-#else
 	  cur.flags = FT_LOAD_TARGET_LCD, subpixel = YES;
-#endif
 //			imgd.type|=|ftc_image_flag_unhinted; /* TODO? when? */
       }
     else
@@ -2877,17 +2801,9 @@ static int filters[3][7]=
 	      continue;
 	    }
 
-#ifdef FT212_STUFF
-	  if (sbit->format == ft_pixel_mode_grays)
-#else
 	  if (sbit->format == FT_PIXEL_MODE_LCD)
-#endif
 	    {
-#ifdef FT212_STUFF
-	      int gx = x + sbit->left, gy = y - sbit->top;
-#else
 	      int gx = 3 * x + sbit->left, gy = y - sbit->top;
-#endif
 	      int px0 = (gx - 2 < 0? gx - 4 : gx - 2) / 3;
 	      int px1 = (gx + sbit->width + 2 < 0? gx + sbit->width + 2: gx + sbit->width + 4) / 3;
 	      int llip = gx - px0 * 3;
@@ -3130,6 +3046,7 @@ static int filters[3][7]=
 	}
     }
 }
+#endif
 
 @end
 
@@ -3143,27 +3060,28 @@ static int filters[3][7]=
 -(NSGlyph) glyphForCharacter: (unichar)ch
 {
   FTFontInfo *fi=fontInfo;
+  NSGlyph g;
 
-  FTC_CMapDescRec cmap;
-
-  cmap.face_id = fi->imgd.font.face_id;
-  cmap.u.encoding = ft_encoding_unicode;
-  cmap.type = FTC_CMAP_BY_ENCODING;
-
-  return FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, ch) + 1;
+  g = FTC_CMapCache_Lookup(ftc_cmapcache, fi->faceId, fi->unicodeCmap, ch);
+  if (g)
+    return g + 1;
+  else
+    return NSNullGlyph;
 }
 
 -(NSString *) nameOfGlyph: (NSGlyph)glyph
 {
   FTFontInfo *fi=fontInfo;
   FT_Face face;
+  FT_Size size;
 
   char buf[256];
 
   glyph--;
 
-  if (FTC_Manager_Lookup_Size(ftc_manager, &fi->imgd.font, &face, 0))
+  if (FTC_Manager_LookupSize(ftc_manager, &fi->scaler, &size))
     return nil;
+  face = size->face;
 
   if (FT_Get_Glyph_Name(face,glyph,buf,sizeof(buf)))
     return nil;
@@ -3229,7 +3147,6 @@ fb04 'ffl'
 	unsigned int ch,ch2,ch3;
 
 	FTFontInfo *fi=[run->font fontInfo];
-	FTC_CMapDescRec cmap;
 
 	NSCharacterSet *cs=[NSCharacterSet controlCharacterSet];
 	IMP characterIsMember=[cs methodForSelector: @selector(characterIsMember:)];
@@ -3241,10 +3158,6 @@ fb04 'ffl'
 	[[_textStorage string] getCharacters: buf
 		range: NSMakeRange(pos,c)];
 
-	cmap.face_id = fi->imgd.font.face_id;
-	cmap.u.encoding = ft_encoding_unicode;
-	cmap.type = FTC_CMAP_BY_ENCODING;
-
 	/* first guess */
 	glyph_size=c;
 	g=run->glyphs=malloc(sizeof(glyph_t)*glyph_size);
@@ -3319,7 +3232,7 @@ fb04 'ffl'
 				ch = 0xfffd;
 		}
 
-		g->g=FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, ch) + 1;
+		g->g=FTC_CMapCache_Lookup(ftc_cmapcache, fi->faceId, fi->unicodeCmap, ch) + 1;
 
 		if (g->g == 1 && ch<0x10000)
 		{
@@ -3335,7 +3248,7 @@ fb04 'ffl'
 					g=run->glyphs+j;
 					memset(&run->glyphs[glyph_size-1],0,sizeof(glyph_t));
 
-					g->g=FTC_CMapCache_Lookup(ftc_cmapcache, &cmap, *decomp) + 1;
+					g->g=FTC_CMapCache_Lookup(ftc_cmapcache, fi->faceId, fi->unicodeCmap, *decomp) + 1;
 					if (g->g == 1)
 						break;
 					c++;
@@ -3372,11 +3285,13 @@ fb04 'ffl'
 -(const char *) nameOfGlyph: (NSGlyph)g
 {
 static char buf[1024]; /* !!TODO!! */
+  FT_Size size;
   FT_Face face;
 
   g--;
-  if (FTC_Manager_Lookup_Size(ftc_manager, &imgd.font, &face, 0))
+  if (FTC_Manager_LookupSize(ftc_manager, &scaler, &size))
     return ".notdef";
+  face = size->face;
 
   if (FT_Get_Glyph_Name(face, g, buf, sizeof(buf)))
     return ".notdef";
@@ -3385,3 +3300,5 @@ static char buf[1024]; /* !!TODO!! */
 }
 @end
 
+#endif /* freetype version check, >=2.1.8 */
+
