@@ -30,6 +30,7 @@
 #include <Foundation/NSRunLoop.h>
 #include <Foundation/NSTimer.h>
 #include <Foundation/NSUserDefaults.h>
+#include <Foundation/NSException.h>
 #include <AppKit/AppKitExceptions.h>
 #include <AppKit/NSApplication.h>
 #include <AppKit/NSGraphics.h>
@@ -48,6 +49,9 @@
 #include <sys/file.h>
 #endif
 
+
+static BOOL handlesWindowDecorations = NO;
+
 static void 
 validateWindow(HWND hwnd, RECT rect);
 LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
@@ -58,7 +62,15 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 /* Initialize AppKit backend */
 + (void)initializeBackend
 {
+  NSUserDefaults	*defs;
+
   NSDebugLog(@"Initializing GNUstep win32 backend.\n");
+  defs = [NSUserDefaults standardUserDefaults];
+  if ([defs objectForKey: @"GSWIN32HandlesWindowDecorations"])
+    {
+      handlesWindowDecorations =
+	[defs boolForKey: @"GSWINHandlesWindowDecorations"];
+    }
   [GSDisplayServer setDefaultServerClass: [WIN32Server class]];
 }
 
@@ -181,10 +193,26 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
   hwnd = WindowFromPoint(p);
   if ((int)hwnd == win)
     {
-      hwnd = GetWindow(hwnd, GW_HWNDNEXT);
+      /*
+       * If the window at the point we want is excluded,
+       * we must look through ALL windows at a lower level
+       * until we find one which contains the same point.
+       */
+      while (hwnd != 0)
+	{
+	  RECT	r;
+
+	  hwnd = GetWindow(hwnd, GW_HWNDNEXT);
+	  GetWindowRect(hwnd, &r);
+	  if (PtInRect(&r, p) && IsWindowVisible(hwnd))
+	    {
+	      break;
+	    }
+	}
     }
 
-  *windowRef = (int)hwnd;
+  *windowRef = (int)hwnd;	// Any windows
+
   return (int)hwnd;
 }
 
@@ -317,12 +345,42 @@ DWORD windowStyleForGSStyle(unsigned int style)
 
 @implementation WIN32Server (WindowOps)
 
+-(BOOL) handlesWindowDecorations
+{
+  return handlesWindowDecorations;
+}
+
+
 - (int) window: (NSRect)frame : (NSBackingStoreType)type : (unsigned int)style
 	      : (int) screen
 {
   HWND hwnd; 
   RECT r;
-  DWORD wstyle = windowStyleForGSStyle(style);
+  DWORD wstyle;
+  DWORD estyle;
+
+  if (handlesWindowDecorations == NO)
+    {
+      wstyle = WS_POPUP;
+    }
+  else
+    {
+      wstyle = windowStyleForGSStyle(style);
+    }
+
+  if ((style & NSMiniaturizableWindowMask)
+    && flags.useWMTaskBar == YES)
+    {
+      estyle = 0;
+    }
+  else
+    {
+      /* 
+       * Not miniaturisable ...
+       * make it a toolwindow so it doesn't appear in the taskbar.
+       */
+      estyle = WS_EX_TOOLWINDOW;
+    }
 
   r = GSScreenRectToMS(frame, style, self);
 
@@ -330,7 +388,7 @@ DWORD windowStyleForGSStyle(unsigned int style)
 	      type, style, screen);
   NSDebugLLog(@"WTrace", @"         device frame: %d, %d, %d, %d", 
 	      r.left, r.top, r.right - r.left, r.bottom - r.top);
-  hwnd = CreateWindowEx(0,
+  hwnd = CreateWindowEx(estyle,
 			"GNUstepWindowClass",
 			"GNUstepWindow",
 			wstyle, 
@@ -338,8 +396,8 @@ DWORD windowStyleForGSStyle(unsigned int style)
 			r.top, 
 			r.right - r.left, 
 			r.bottom - r.top,
-			(HWND) NULL,
-			(HMENU) NULL,
+			(HWND)NULL,
+			(HMENU)NULL,
 			hinstance,
 			(void*)type);
   NSDebugLLog(@"WTrace", @"         num/handle: %d", hwnd);
@@ -358,6 +416,9 @@ DWORD windowStyleForGSStyle(unsigned int style)
 {
   DWORD wstyle = windowStyleForGSStyle(style);
 
+  NSAssert(handlesWindowDecorations,
+    @"-stylewindow:: called when handlesWindowDecorations==NO");
+
   NSDebugLLog(@"WTrace", @"stylewindow: %d : %d", style, winNum);
   SetWindowLong((HWND)winNum, GWL_STYLE, wstyle);
 }
@@ -617,17 +678,28 @@ DWORD windowStyleForGSStyle(unsigned int style)
 - (void) styleoffsets: (float *) l : (float *) r : (float *) t : (float *) b
 		     : (unsigned int) style 
 {
-  DWORD wstyle = windowStyleForGSStyle(style);
-  RECT rect = {100, 100, 200, 200};
-  
-  AdjustWindowRectEx(&rect, wstyle, NO, 0);
+  if (handlesWindowDecorations)
+    {
+      DWORD wstyle = windowStyleForGSStyle(style);
+      RECT rect = {100, 100, 200, 200};
+      
+      AdjustWindowRectEx(&rect, wstyle, NO, 0);
 
-  *l = 100 - rect.left;
-  *r = rect.right - 200;
-  *t = 100 - rect.top;
-  *b = rect.bottom - 200;
-
-  //NSLog(@"Sytle %d offset %f %f %f %f", wstyle, *l, *r, *t, *b);
+      *l = 100 - rect.left;
+      *r = rect.right - 200;
+      *t = 100 - rect.top;
+      *b = rect.bottom - 200;
+      //NSLog(@"Style %d offset %f %f %f %f", wstyle, *l, *r, *t, *b);
+    }
+  else
+    {
+      /*
+      If we don't handle decorations, all our windows are going to be
+      border- and decorationless. In that case, -gui won't call this method,
+      but we still use it internally.
+      */
+      *l = *r = *t = *b = 0.0;
+    }
 }
 
 - (void) docedited: (int) edited : (int) winNum
@@ -636,6 +708,10 @@ DWORD windowStyleForGSStyle(unsigned int style)
 
 - (void) setinputstate: (int)state : (int)winNum
 {
+  if (handlesWindowDecorations == NO)
+    {
+      return;
+    }
   if (state == GSTitleBarKey)
     {
       SetActiveWindow((HWND)winNum);
@@ -706,7 +782,7 @@ DWORD windowStyleForGSStyle(unsigned int style)
 
 - (void) standardcursor: (int)style : (void **)cid
 {
-  HCURSOR hCursor;
+  HCURSOR hCursor = 0;
 
   NSDebugLLog(@"WTrace", @"standardcursor: %d", style);
   switch (style)
@@ -730,14 +806,13 @@ DWORD windowStyleForGSStyle(unsigned int style)
       hCursor = LoadCursor(NULL, IDC_SIZENS);
       break;
     default:
-      hCursor = LoadCursor(NULL, IDC_ARROW);
-      break;
+      return;
     }
   *cid = (void*)hCursor;
 }
 
 - (void) imagecursor: (NSPoint)hotp : (int) w :  (int) h 
-		    : (int)colors : (const char *)image : (void **)cid
+		    : (int)colors : (const unsigned char *)image : (void **)cid
 {
   /*
   HCURSOR cur;
