@@ -84,6 +84,7 @@ static KeySym _command_keysyms[2];
 static KeySym _alt_keysyms[2];
 
 static BOOL _is_keyboard_initialized = NO;
+static BOOL _mod_ignore_shift = NO;
 
 void __objc_xgcontextevent_linking (void)
 {
@@ -127,42 +128,28 @@ static void initialize_keyboard (void);
 
 static void set_up_num_lock (void);
 
-// checks if given keycode is set in bit vector
-static inline int check_key (XEvent *xEvent, KeyCode key_code) 
-{
-    return (key_code == 0) ?
-        0 : (xEvent->xkeymap.key_vector[key_code / 8] & (1 << (key_code % 8)));
-}
-
-// checks whether a GNUstep modifier is pressed when we're only able to check
-// whether X keycodes are pressed
+// checks whether a GNUstep modifier (key_sym) is pressed when we're only able
+// to check whether X keycodes are pressed in xEvent->xkeymap;
 static int check_modifier (XEvent *xEvent, KeySym key_sym,
                            XModifierKeymap *modmap)
 {
-  int m;
-  int c;
-  KeyCode key_code;
+    char *key_vector;
+    int by,bi;
 
-  // if key_sym is a modifier, check each of its keycodes
-  for (m=0; m<8; m++)
-    {
-      key_code = modmap->modifiermap[m * modmap->max_keypermod];
-      if ((key_code != 0)
-          && XKeycodeToKeysym(xEvent->xkeymap.display, key_code, 0) == key_sym)
-        {
-          for (c=0; c<modmap->max_keypermod; c++)
-            {
-              if (check_key(xEvent,
-                            modmap->modifiermap[m * modmap->max_keypermod + c]))
+    // implementation fails if key_sym is only accessible by a multi-key
+    // combination, however this is not expected for GNUstep modifier keys;
+    // to fix implementation, would need to first determine modifier state by
+    // a first-pass scann of the full key vector, then use the state in place
+    // of "0" below in XKeycodeToKeysym
+    key_vector = xEvent->xkeymap.key_vector;
+    for (by=0; by<32; by++) {
+        for (bi=0; bi<8; bi++) {
+            if ((key_vector[by] & (1 << bi)) &&
+                (XKeycodeToKeysym(xEvent->xkeymap.display,by*8+bi,0)==key_sym))
                 return 1;
-            }
-          return 0; // no dice
         }
     }
-  // wasn't a modifier; just check the first keycode for this keysym,
-  // which ignores other possibilities but that's the best we can do
-  // w/o XtKeysymToKeycodeList
-  return check_key(xEvent, XKeysymToKeycode(xEvent->xkeymap.display, key_sym));
+    return 0;
 }
 
 @implementation XGServer (EventOps)
@@ -584,9 +571,23 @@ static int check_modifier (XEvent *xEvent, KeySym key_sym,
 		Window		source;
 		Atom		action;
 		NSDragOperation	operation;
+                int root_x, root_y;
+                Window root_child;
 
 		NSDebugLLog(@"NSDragging", @"  XdndPosition message\n");
 		source = XDND_POSITION_SOURCE_WIN(&xEvent);
+                /*
+                  Work around a bug/feature in WindowMaker that does not
+                  send ConfigureNotify events for app icons.
+                */
+                XTranslateCoordinates(dpy, xEvent.xclient.window,
+                                      RootWindow(dpy, cWin->screen),
+                                      0, 0,
+                                      &root_x, &root_y,
+                                      &root_child);
+                cWin->xframe.origin.x = root_x;
+                cWin->xframe.origin.y = root_y;
+
 		eventLocation.x = XDND_POSITION_ROOT_X(&xEvent) - 
 		  NSMinX(cWin->xframe);
 		eventLocation.y = XDND_POSITION_ROOT_Y(&xEvent) - 
@@ -1240,8 +1241,8 @@ static int check_modifier (XEvent *xEvent, KeySym key_sym,
 		while (new_parent && (new_parent != cWin->root)) {
 		  Window root;
 		  Window *children;
-		  int nchildren;
-	    
+		  unsigned int nchildren;
+
 		  parent = new_parent;
 		  NSLog(@"QueryTree window is %d (root %d cwin root %d)", 
 			parent, root, cWin->root);
@@ -1462,7 +1463,7 @@ initialize_keyboard (void)
     _alt_keysyms[0] = XK_Mode_switch;
 
   _alt_keysyms[1] = key_sym_from_defaults(display, defaults,
-                                          @"GSecondAlternateKey",
+                                          @"GSSecondAlternateKey",
                                           NoSymbol);
 
   if (_alt_keysyms[0] == _alt_keysyms[1])
@@ -1470,6 +1471,7 @@ initialize_keyboard (void)
 
   
   set_up_num_lock ();
+  _mod_ignore_shift = [defaults boolForKey: @"GSModifiersAreKeys"];
   
   _is_keyboard_initialized = YES;
 }
@@ -1550,37 +1552,63 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType)
   int		control_key = 0;
   int		command_key = 0;
   int		alt_key = 0;
+  KeySym	modKeysym;  // process modifier independently of shift, etc.
   
   if (_is_keyboard_initialized == NO)
     initialize_keyboard ();
 
-  /* Process NSFlagsChanged events.  We can't use a switch because we
-     are not comparing to constants. Make sure keyCode is not 0 since
-     XIM events can potentially return 0 keyCodes. */
-  keysym = XLookupKeysym((XKeyEvent *)xEvent, 0);
-  if (keysym != NoSymbol)
+  /* Process location */
+  window = [XGServer _windowWithTag: [[NSApp keyWindow] windowNumber]];
+  eventLocation.x = xEvent->xbutton.x;
+  if (window)
     {
-      if (keysym == _control_keysyms[0]) 
+      eventLocation.y = window->siz_hints.height - xEvent->xbutton.y;
+    }
+  else
+    {
+      eventLocation.y = xEvent->xbutton.y;
+    }
+    
+  /* Process characters */
+  keys = [context->inputServer lookupStringForEvent: (XKeyEvent *)xEvent
+		 window: window
+		 keysym: &keysym];
+
+  /* Process keycode */
+  keyCode = ((XKeyEvent *)xEvent)->keycode;
+  //ximKeyCode = XKeysymToKeycode([XGServer currentXDisplay],keysym);
+
+  /* Process NSFlagsChanged events.  We can't use a switch because we
+     are not comparing to constants. Make sure keySym is not NoSymbol since
+     XIM events can potentially return this. */
+  /* Possibly ignore shift/other modifier state in determining KeySym to
+     work around correct but undesired behavior with shifted modifiers.
+     See Back defaults documentation for "GSModifiersAreKeys". */
+  modKeysym = (_mod_ignore_shift == YES) ?
+      XLookupKeysym((XKeyEvent *)xEvent, 0) : keysym;
+  if (modKeysym != NoSymbol)
+    {
+      if (modKeysym == _control_keysyms[0]) 
 	{
 	  control_key = 1;
 	}
-      else if (keysym == _control_keysyms[1])
+      else if (modKeysym == _control_keysyms[1])
 	{
 	  control_key = 2;
 	}
-      else if (keysym == _command_keysyms[0]) 
+      else if (modKeysym == _command_keysyms[0]) 
 	{
 	  command_key = 1;
 	}
-      else if (keysym == _command_keysyms[1]) 
+      else if (modKeysym == _command_keysyms[1]) 
 	{
 	  command_key = 2;
 	}
-      else if (keysym == _alt_keysyms[0]) 
+      else if (modKeysym == _alt_keysyms[0]) 
 	{
 	  alt_key = 1;
 	}
-      else if (keysym == _alt_keysyms[1]) 
+      else if (modKeysym == _alt_keysyms[1]) 
 	{
 	  alt_key = 2;
 	}
@@ -1613,27 +1641,6 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType)
   /* Process modifiers */
   eventFlags = process_modifier_flags (xEvent->xkey.state);
 
-  /* Process location */
-  window = [XGServer _windowWithTag: [[NSApp keyWindow] windowNumber]];
-  eventLocation.x = xEvent->xbutton.x;
-  if (window)
-    {
-      eventLocation.y = window->siz_hints.height - xEvent->xbutton.y;
-    }
-  else
-    {
-      eventLocation.y = xEvent->xbutton.y;
-    }
-    
-  /* Process characters */
-  keys = [context->inputServer lookupStringForEvent: (XKeyEvent *)xEvent
-		 window: window
-		 keysym: &keysym];
-
-  /* Process keycode */
-  keyCode = ((XKeyEvent *)xEvent)->keycode;
-  //ximKeyCode = XKeysymToKeycode([XGServer currentXDisplay],keysym);
-
   /* Add NSNumericPadKeyMask if the key is in the KeyPad */
   if (IsKeypadKey (keysym))
     eventFlags = eventFlags | NSNumericPadKeyMask;
