@@ -140,7 +140,34 @@ setNormalHints(Display *d, gswindow_device_t *w)
   if (w->siz_hints.flags & PResizeInc)
     NSDebugLLog(@"XGTrace", @"Hint incr %d: %d, %d",
       w->number, w->siz_hints.width_inc, w->siz_hints.height_inc);
-  XSetWMNormalHints(d, w->ident, &w->siz_hints);
+  if (handlesWindowDecorations
+      && !(w->win_attrs.window_style & NSResizableWindowMask))
+    {
+      /* Some silly window managers (*cough* metacity *cough*) ignore
+	 our "non-resizable" hints unless we set the min and max
+	 sizes equal to the current size, hence the ugly code here.  */
+      CARD32 oldFlags;
+      int old_w0, old_h0, old_w1, old_h1;
+      
+      old_w0 = w->siz_hints.min_width;
+      old_h0 = w->siz_hints.max_width;
+      old_w1 = w->siz_hints.min_height;
+      old_h1 = w->siz_hints.max_height;
+      oldFlags = w->siz_hints.flags;
+
+      w->siz_hints.flags |= PMinSize | PMaxSize;
+      w->siz_hints.min_width = w->siz_hints.max_width = w->xframe.size.width;
+      w->siz_hints.min_height = w->siz_hints.max_height = w->xframe.size.height;
+      XSetWMNormalHints(d, w->ident, &w->siz_hints);
+
+      w->siz_hints.min_width = old_w0;
+      w->siz_hints.max_width = old_h0;
+      w->siz_hints.min_height = old_w1;
+      w->siz_hints.max_height = old_h1;
+      w->siz_hints.flags = oldFlags;
+    }
+  else
+    XSetWMNormalHints(d, w->ident, &w->siz_hints);
 }
 
 /*
@@ -195,10 +222,6 @@ typedef struct {
 #define _XA_MOTIF_WM_HINTS "_MOTIF_WM_HINTS"
 
 
-Pixmap
-xgps_cursor_mask(Display *xdpy, Drawable draw, const char *data,
-                  int w, int h, int colors);
-
 /* Now the code */
 
 /* Set the style `styleMask' for the XWindow `window' using motif
@@ -586,7 +609,8 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 
 - (gswindow_device_t *)_rootWindowForScreen: (int)screen
 {
-  int x, y, width, height;
+  int x, y;
+  unsigned int width, height;
   gswindow_device_t *window;
 
   /* Screen number is negative to avoid conflict with windows */
@@ -1500,7 +1524,7 @@ static BOOL didCreatePixmaps;
   data = [rep bitmapData];
   screen = [[[self screenList] objectAtIndex: 0] intValue];
   xIconPixmap = XCreatePixmap(dpy,
-                      [self xDisplayRootWindowForScreen:screen],
+                      [self xDisplayRootWindowForScreen: screen],
                       [rep pixelsWide], [rep pixelsHigh],
                       DefaultDepth(dpy, screen));
   pixgc = XCreateGC(dpy, xIconPixmap, 0, NULL);
@@ -1855,7 +1879,6 @@ static BOOL didCreatePixmaps;
   XMoveResizeWindow (dpy, window->ident,
     window->siz_hints.x, window->siz_hints.y,
     window->siz_hints.width, window->siz_hints.height);
-  setNormalHints(dpy, window);
 
   /* Update xframe right away. We optimistically assume that we'll get the
   frame we asked for. If we're right, -gui can update/redraw right away,
@@ -1864,6 +1887,11 @@ static BOOL didCreatePixmaps;
   at that point, we'll send new GSAppKitWindow* events to -gui. */
   window->xframe = xVal;
 
+  /* Update the hints.  Note that we do this _after_ updating xframe since
+     the hint setting code needs the new xframe to work around problems
+     with min/max sizes and resizability in some window managers.  */
+  setNormalHints(dpy, window);
+
   if (resize == YES)
     {
       NSDebugLLog(@"Moving", @"Fake size %d - %@", window->number,
@@ -1988,10 +2016,15 @@ static BOOL didCreatePixmaps;
 		   level == NSFloatingWindowLevel ||
 		   level == NSTornOffMenuWindowLevel)
 	    {
+#ifdef USE_KDE_OVERRIDE
 	      data[0] = generic.wintypes.win_override_atom;
 	      //data[0] = generic.wintypes.win_utility_atom;
 	      data[1] = generic.wintypes.win_menu_atom;
 	      len = 2;
+#else
+	      data[0] = generic.wintypes.win_menu_atom;
+	      len = 1;
+#endif
 	    }
 	  else if (level == NSDockWindowLevel ||
 		   level == NSStatusWindowLevel)
@@ -2001,9 +2034,14 @@ static BOOL didCreatePixmaps;
 	  // Does this belong into a different category?
 	  else if (level == NSPopUpMenuWindowLevel)
 	    {
+#ifdef USE_KDE_OVERRIDE
 	      data[0] = generic.wintypes.win_override_atom;
 	      data[1] = generic.wintypes.win_floating_atom;
 	      len = 2;
+#else
+	      data[0] = generic.wintypes.win_floating_atom;
+	      len = 1;
+#endif
 	    }
 	  else if (level == NSDesktopWindowLevel)
 	    {
@@ -2396,6 +2434,46 @@ static BOOL didCreatePixmaps;
     }
 }
 
+/** Sets the transparancy value for the whole window */
+- (void) setalpha: (float)alpha: (int) win
+{
+  gswindow_device_t *window = WINDOW_WITH_TAG(win);
+  static Atom opacity_atom = None;
+
+  if (win == 0 || window == 0)
+    {
+      NSDebugLLog(@"XGTrace", @"Setting alpha to unknown win %d", win);
+      return;
+    }
+
+  NSDebugLLog(@"XGTrace", @"setalpha: %d", win);
+  
+  /* Initialize the atom if needed */
+  if (opacity_atom == None)
+    opacity_atom = XInternAtom (window->display, "_NET_WM_WINDOW_OPACITY", False);
+
+  if (alpha == 1.0)
+    {
+      XDeleteProperty(window->display, window->ident, opacity_atom);
+    }
+  else
+    {
+      unsigned int opacity;
+
+      opacity = (unsigned int)(alpha * 0xffffffff);
+      XChangeProperty(window->display, window->ident, opacity_atom,
+		      XA_CARDINAL, 32, PropModeReplace,
+		      (unsigned char*)&opacity, 1L);
+      if (window->parent != window->root)
+        {
+	  XChangeProperty(window->display, window->parent, opacity_atom,
+			  XA_CARDINAL, 32, PropModeReplace,
+			  (unsigned char*)&opacity, 1L);
+	}
+    }
+}
+
+
 - (void *) serverDevice
 {
   return dpy;
@@ -2415,9 +2493,6 @@ static BOOL didCreatePixmaps;
 }
 
 /* Cursor Ops */
-typedef struct _xgps_cursor_id_t {
-  Cursor c;
-} xgps_cursor_id_t;
 
 static char xgps_blank_cursor_bits [] = {
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -2467,9 +2542,9 @@ static BOOL   cursor_hidden = NO;
     }
   else
     {
-      xgps_cursor_id_t *cid = [[NSCursor currentCursor] _cid];
+      Cursor cid = (Cursor)[[NSCursor currentCursor] _cid];
       
-      XDefineCursor (dpy, win, cid->c);
+      XDefineCursor (dpy, win, cid);
     }
 }
 
@@ -2503,7 +2578,7 @@ static BOOL   cursor_hidden = NO;
 #define ALPHA_THRESHOLD 158
 
 Pixmap
-xgps_cursor_mask(Display *xdpy, Drawable draw, const char *data, 
+xgps_cursor_mask(Display *xdpy, Drawable draw, const unsigned char *data,
 		  int w, int h, int colors)
 {
   int j, i;
@@ -2650,54 +2725,54 @@ xgps_cursor_image(Display *xdpy, Drawable draw, const unsigned char *data,
 
 - (void) standardcursor: (int)style : (void **)cid
 {
-  xgps_cursor_id_t *cursor;
-  cursor = NSZoneMalloc([self zone], sizeof(xgps_cursor_id_t));
+  Cursor cursor = None;
+
   switch (style)
     {
     case GSArrowCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_left_ptr);     
+      cursor = XCreateFontCursor(dpy, XC_left_ptr);     
       break;
     case GSIBeamCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_xterm);
+      cursor = XCreateFontCursor(dpy, XC_xterm);
       break;
     case GSCrosshairCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_crosshair);
+      cursor = XCreateFontCursor(dpy, XC_crosshair);
       break;
     case GSDisappearingItemCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_shuttle);
+      cursor = XCreateFontCursor(dpy, XC_shuttle);
       break;
     case GSPointingHandCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_hand1);
+      cursor = XCreateFontCursor(dpy, XC_hand1);
       break;
     case GSResizeDownCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_bottom_side);
+      cursor = XCreateFontCursor(dpy, XC_bottom_side);
       break;
     case GSResizeLeftCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_left_side);
+      cursor = XCreateFontCursor(dpy, XC_left_side);
       break;
     case GSResizeLeftRightCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_sb_h_double_arrow);
+      cursor = XCreateFontCursor(dpy, XC_sb_h_double_arrow);
       break;
     case GSResizeRightCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_right_side);
+      cursor = XCreateFontCursor(dpy, XC_right_side);
       break;
     case GSResizeUpCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_top_side);
+      cursor = XCreateFontCursor(dpy, XC_top_side);
       break;
     case GSResizeUpDownCursor:
-      cursor->c = XCreateFontCursor(dpy, XC_sb_v_double_arrow);
+      cursor = XCreateFontCursor(dpy, XC_sb_v_double_arrow);
       break;
     default:
-      cursor->c = XCreateFontCursor(dpy, XC_left_ptr);     
-      break;
+      return;
     }
   if (cid)
     *cid = (void *)cursor;
 }
 
-- (void) imagecursor: (NSPoint)hotp : (int) w :  (int) h : (int)colors : (const char *)image : (void **)cid
+- (void) imagecursor: (NSPoint)hotp : (int) w :  (int) h : (int)colors
+		    : (const unsigned char *)image : (void **)cid
 {
-  xgps_cursor_id_t *cursor;
+  Cursor cursor;
   Pixmap source, mask;
   unsigned int maxw, maxh;
   XColor fg, bg;
@@ -2716,14 +2791,13 @@ xgps_cursor_image(Display *xdpy, Drawable draw, const unsigned char *data,
   if ((unsigned int)h > maxh)
     h = maxh;
 
-  cursor = NSZoneMalloc([self zone], sizeof(xgps_cursor_id_t));
   source = xgps_cursor_image(dpy, ROOT, image, w, h, colors, &fg, &bg);
   mask = xgps_cursor_mask(dpy, ROOT, image, w, h, colors);
   bg = [self xColorFromColor: bg forScreen: defScreen];
   fg = [self xColorFromColor: fg forScreen: defScreen];
 
-  cursor->c = XCreatePixmapCursor(dpy, source, mask, &fg, &bg, 
-				  (int)hotp.x, (int)(h - hotp.y));
+  cursor = XCreatePixmapCursor(dpy, source, mask, &fg, &bg, 
+			       (int)hotp.x, (int)(h - hotp.y));
   XFreePixmap(dpy, source);
   XFreePixmap(dpy, mask);
   if (cid)
@@ -2733,13 +2807,13 @@ xgps_cursor_image(Display *xdpy, Drawable draw, const unsigned char *data,
 - (void) setcursorcolor: (NSColor *)fg : (NSColor *)bg : (void*) cid
 {
   XColor xf, xb;
-  xgps_cursor_id_t *cursor;
+  Cursor cursor;
 
-  cursor = (xgps_cursor_id_t *)cid;
-  if (cursor == NULL)
+  cursor = (Cursor)cid;
+  if (cursor == None)
     NSLog(@"Invalidparam: Invalid cursor");
 
-  [self _DPSsetcursor: cursor->c : YES];
+  [self _DPSsetcursor: cursor : YES];
   /* Special hack: Don't set the color when fg == nil. Used by NSCursor
      to just set the cursor but not the color. */
   if (fg == nil)
@@ -2758,7 +2832,7 @@ xgps_cursor_image(Display *xdpy, Drawable draw, const unsigned char *data,
   xf = [self xColorFromColor: xf forScreen: defScreen];
   xb = [self xColorFromColor: xb forScreen: defScreen];
 
-  XRecolorCursor(dpy, cursor->c, &xf, &xb);
+  XRecolorCursor(dpy, cursor, &xf, &xb);
 }
 
 static NSWindowDepth
@@ -2893,5 +2967,84 @@ _computeDepth(int class, int bpp)
 		   DisplayHeight(dpy, screen));
 }
 
+- (NSImage *) iconTileImage
+{
+  Atom noticeboard_atom;
+  Atom icon_tile_atom;
+  Atom rgba_image_atom;
+  Window win;
+  int count;
+  unsigned char *tile;
+  NSImage *iconTileImage;
+  NSBitmapImageRep *imageRep;
+  unsigned int width, height;
+
+  if (((generic.wm & XGWM_WINDOWMAKER) == 0)
+      || generic.flags.useWindowMakerIcons == NO)
+    return [super iconTileImage];
+
+  noticeboard_atom = XInternAtom(dpy,"_WINDOWMAKER_NOTICEBOARD", False);
+  icon_tile_atom = XInternAtom(dpy,"_WINDOWMAKER_ICON_TILE", False);
+  rgba_image_atom = XInternAtom(dpy,"_RGBA_IMAGE", False);
+  
+  win = DefaultRootWindow(dpy);
+  win = *(Window *)PropGetCheckProperty(dpy, win, noticeboard_atom, XA_WINDOW,
+					32, -1, &count);
+  if (win == (Window)NULL) 
+    return [super iconTileImage];
+  
+  tile = PropGetCheckProperty(dpy, win, icon_tile_atom, rgba_image_atom,
+			      8, -1, &count);
+  if (tile == NULL || count < 4)
+    return [super iconTileImage];
+  
+  width = (tile[0] << 8) + tile[1];
+  height = (tile[2] << 8) + tile[3];
+  
+  if (count > 4 + width * height * 4)
+    return [super iconTileImage];
+  
+  iconTileImage = [[NSImage alloc] init];
+  imageRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes: NULL
+                                                     pixelsWide: width 
+                                                     pixelsHigh: height 
+                                                  bitsPerSample: 8
+                                                samplesPerPixel: 4
+                                                       hasAlpha: YES
+                                                       isPlanar: NO
+                                                 colorSpaceName: NSDeviceRGBColorSpace
+                                                    bytesPerRow: width * 4
+                                                   bitsPerPixel: 32];
+  memcpy([imageRep bitmapData], &tile[4], width * height * 4);
+  XFree(tile);
+  [iconTileImage addRepresentation:imageRep];
+  RELEASE(imageRep);
+
+  return AUTORELEASE(iconTileImage);
+}
+
+- (NSSize) iconSize
+{
+  XIconSize *xiconsize;
+  int count_return;
+  int status;
+  
+  status = XGetIconSizes(dpy,
+                         DefaultRootWindow(dpy),
+			 &xiconsize,
+			 &count_return);
+  if (status)
+    {
+      if ((generic.wm & XGWM_WINDOWMAKER) != 0) 
+        {
+	  /* must add 4 pixels for the border which windowmaker leaves out
+	     but gnustep draws over. */
+          return NSMakeSize(xiconsize[0].max_width + 4,
+			    xiconsize[0].max_height + 4);
+	}
+    }
+  return [super iconSize];
+}
+
 @end
 
