@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
    Author:  Alexander Malmberg <alexander@malmberg.org>
 
@@ -41,8 +41,121 @@ static int num_window_buffers;
 
 static int use_shape_hack = 0; /* this is an ugly hack :) */
 
-static int use_xshm = 1;
 
+static int did_test_xshm = 0;
+static int use_xshm = 1;
+static int num_xshm_test_errors = 0;
+
+static NSString *xshm_warning
+	= @"Falling back to normal XImage:s (will be slower).";
+
+static int test_xshm_error_handler(Display *d, XErrorEvent *ev)
+{
+  num_xshm_test_errors++;
+  return 0;
+}
+
+static void test_xshm(Display *display, int drawing_depth)
+{
+  NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
+
+  did_test_xshm = 1;
+
+  if ([ud objectForKey: @"XWindowBufferUseXShm"])
+    use_xshm = [ud boolForKey: @"XWindowBufferUseXShm"];
+
+  /* If the user doesn't want XShm, there's no point in doing any further
+  testing. */
+  if (!use_xshm)
+    return;
+
+  /* This seems to return success if the X server understands the XShm
+  protocol, regardless of whether XShm can actually be used or not... */
+  if (!XShmQueryExtension(display))
+    {
+      NSLog(@"XShm not supported by X server.");
+      NSLog(xshm_warning);
+      use_xshm = 0;
+      return;
+    }
+
+  /* ... so we check that it actually works here. To do this, we need to
+  set up our own error handler (because the xlib calls never fail, they
+  just cause error events to be sent to us), explicitly synchronize
+  (or we wouldn't get to handle, and thus detect, the errors until much
+  later), and try actually creating a XShm image. Yuck. */
+  {
+    XImage *ximage;
+    XShmSegmentInfo shminfo;
+    int (*old_error_handler)();
+
+    old_error_handler = XSetErrorHandler(test_xshm_error_handler);
+
+    ximage = XShmCreateImage(display,
+	DefaultVisual(display, DefaultScreen(display)),
+	drawing_depth, ZPixmap, NULL, &shminfo,
+	1, 1);
+    if (!ximage || num_xshm_test_errors)
+      {
+	NSLog(@"XShm not supported, XShmCreateImage failed.");
+	goto no_xshm;
+      }
+
+    shminfo.shmid = shmget(IPC_PRIVATE,
+	64, /* We don't have exact bytes per row values here, but this
+	    should be safe, and we'll probably get a full page anyway.
+ 	    (And if it turns out not to be enough, the X server will notice
+	    and give us errors, causing us to think that XShm isn't
+	    supported.) */
+	IPC_CREAT | 0700);
+
+    if (shminfo.shmid == -1 || num_xshm_test_errors)
+      {
+	NSLog(@"XShm not supported, shmget() failed: %m.");
+	XDestroyImage(ximage);
+	goto no_xshm;
+      }
+
+    shminfo.shmaddr = shmat(shminfo.shmid, 0, 0);
+    if ((int)shminfo.shmaddr == -1 || num_xshm_test_errors)
+      {
+	NSLog(@"XShm not supported, shmat() failed: %m.");
+	XDestroyImage(ximage);
+	shmctl(shminfo.shmid, IPC_RMID, 0);
+	goto no_xshm;
+      }
+
+    shminfo.readOnly = 0;
+    if (!XShmAttach(display, &shminfo) || num_xshm_test_errors)
+      {
+	NSLog(@"XShm not supported, XShmAttach() failed.");
+	XDestroyImage(ximage);
+	shmdt(shminfo.shmaddr);
+	shmctl(shminfo.shmid, IPC_RMID, 0);
+	goto no_xshm;
+      }
+
+    XShmDetach(display, &shminfo);
+    XDestroyImage(ximage);
+    shmdt(shminfo.shmaddr);
+
+    /* Most of the time, all the above calls will return success despite
+    actually failing. To catch all errors generated by the calls before
+    returning, we synchronize here. */
+    XSync(display, False);
+
+    shmctl(shminfo.shmid, IPC_RMID, 0);
+
+    if (num_xshm_test_errors)
+      {
+	NSLog(@"XShm not supported.");
+no_xshm:
+	NSLog(xshm_warning);
+	use_xshm = 0;
+      }
+    XSetErrorHandler(old_error_handler);
+  }
+}
 
 
 @implementation XWindowBuffer
@@ -149,20 +262,13 @@ static int use_xshm = 1;
       if (wi->window->xframe.size.width * wi->window->xframe.size.height < 4096)
         goto no_xshm;
 
-#define WARN @" Falling back to normal XImage:s (will be slower)."
+      if (!did_test_xshm)
+	test_xshm(wi->display, aDI->drawing_depth);
+
       if (!use_xshm)
         goto no_xshm;
 
       /* Use XShm if possible, else fall back to normal XImage:s */
-      if (!XShmQueryExtension(wi->display))
-	{
-static BOOL xshm_warned = NO;
-	  if (!xshm_warned)
-	    NSLog(@"XShm not supported." WARN);
-	  xshm_warned = YES;
-	  goto no_xshm;
-	}
-
       wi->use_shm = 1;
       wi->ximage = XShmCreateImage(wi->display,
 	DefaultVisual(wi->display, DefaultScreen(wi->display)),
@@ -171,7 +277,8 @@ static BOOL xshm_warned = NO;
 	wi->window->xframe.size.height);
       if (!wi->ximage)
 	{
-	  NSLog(@"Warning: XShmCreateImage failed!" WARN);
+	  NSLog(@"Warning: XShmCreateImage failed!");
+	  NSLog(xshm_warning);
 	  goto no_xshm;
 	}
       wi->shminfo.shmid = shmget(IPC_PRIVATE,
@@ -180,7 +287,8 @@ static BOOL xshm_warned = NO;
 
       if (wi->shminfo.shmid == -1)
 	{
-	  NSLog(@"Warning: shmget() failed: %m." WARN);
+	  NSLog(@"Warning: shmget() failed: %m.");
+	  NSLog(xshm_warning);
 	  XDestroyImage(wi->ximage);
 	  goto no_xshm;
 	}
@@ -188,7 +296,8 @@ static BOOL xshm_warned = NO;
       wi->shminfo.shmaddr = wi->ximage->data = shmat(wi->shminfo.shmid, 0, 0);
       if ((int)wi->shminfo.shmaddr == -1)
 	{
-	  NSLog(@"Warning: shmat() failed: %m." WARN);
+	  NSLog(@"Warning: shmat() failed: %m.");
+	  NSLog(xshm_warning);
 	  XDestroyImage(wi->ximage);
 	  shmctl(wi->shminfo.shmid, IPC_RMID, 0);
 	  goto no_xshm;
@@ -197,7 +306,8 @@ static BOOL xshm_warned = NO;
       wi->shminfo.readOnly = 0;
       if (!XShmAttach(wi->display, &wi->shminfo))
 	{
-	  NSLog(@"Warning: XShmAttach() failed." WARN);
+	  NSLog(@"Warning: XShmAttach() failed.");
+	  NSLog(xshm_warning);
 	  XDestroyImage(wi->ximage);
 	  shmdt(wi->shminfo.shmaddr);
 	  shmctl(wi->shminfo.shmid, IPC_RMID, 0);
@@ -210,20 +320,21 @@ static BOOL xshm_warned = NO;
 	 dragged across a window. */
       /* TODO: we still get and handle expose events, although we don't
 	 need to. */
-      wi->pixmap=XShmCreatePixmap(wi->display,wi->drawable,
-				  wi->ximage->data,&wi->shminfo,
-				  wi->window->xframe.size.width,
-				  wi->window->xframe.size.height,
-				  aDI->drawing_depth);
+      wi->pixmap = XShmCreatePixmap(wi->display, wi->drawable,
+				    wi->ximage->data, &wi->shminfo,
+				    wi->window->xframe.size.width,
+				    wi->window->xframe.size.height,
+				    aDI->drawing_depth);
       if (wi->pixmap) /* TODO: this doesn't work */
 	{
-	  XSetWindowBackgroundPixmap(wi->display,wi->window->ident,wi->pixmap);
+	  XSetWindowBackgroundPixmap(wi->display, wi->window->ident,
+				     wi->pixmap);
 	}
 
       /* On some systems (eg. freebsd), X can't attach to the shared segment
       if it's marked for destruction, so we make sure it's attached before
       marking it. */
-      XSync(wi->display,False);
+      XSync(wi->display, False);
 
       /* Mark the segment as destroyed now. Since we're attached, it won't
       actually be destroyed, but if we crashed before doing this, it wouldn't
@@ -255,7 +366,7 @@ no_xshm:
     {
       wi->sx = wi->ximage->width;
       wi->sy = wi->ximage->height;
-      wi->data = wi->ximage->data;
+      wi->data = (unsigned char *)wi->ximage->data;
       wi->bytes_per_line = wi->ximage->bytes_per_line;
       wi->bits_per_pixel = wi->ximage->bits_per_pixel;
       wi->bytes_per_pixel = wi->bits_per_pixel / 8;
@@ -281,24 +392,23 @@ extern int XShmGetEventBase(Display *d);
   pending_event = 0;
   if (pending_put)
     {
-      NSRect r = pending_rect;
       pending_put = 0;
-      if (r.origin.x + r.size.width>window->xframe.size.width)
+      if (pending_rect.x + pending_rect.w > window->xframe.size.width)
 	{
-	  r.size.width = window->xframe.size.width - r.origin.x;
-	  if (r.size.width <= 0)
+	  pending_rect.w = window->xframe.size.width - pending_rect.x;
+	  if (pending_rect.w <= 0)
 	    return;
 	}
-      if (r.origin.y + r.size.height>window->xframe.size.height)
+      if (pending_rect.y + pending_rect.h > window->xframe.size.height)
 	{
-	  r.size.height = window->xframe.size.height - r.origin.y;
-	  if (r.size.height <= 0)
+	  pending_rect.h = window->xframe.size.height - pending_rect.y;
+	  if (pending_rect.h <= 0)
 	    return;
 	}
       if (!XShmPutImage(display, drawable, gc, ximage,
-			r.origin.x, r.origin.y,
-			r.origin.x, r.origin.y,
-			r.size.width, r.size.height,
+			pending_rect.x, pending_rect.y,
+			pending_rect.x, pending_rect.y,
+			pending_rect.w, pending_rect.h,
 			1))
 	{
 	  NSLog(@"XShmPutImage failed?");
@@ -311,7 +421,7 @@ extern int XShmGetEventBase(Display *d);
 //	XFlush(window->display);
 }
 
--(void) _exposeRect: (NSRect)r
+-(void) _exposeRect: (NSRect)rect
 {
 /* TODO: Somehow, we can get negative coordinates in the rectangle. So far
 I've tracked them back to [NSWindow flushWindow]. Should probably figure
@@ -323,21 +433,39 @@ Also, just about every resize of a window causes a few calls here with
 rects in the new size before we are updated.
 
 For now, we just intersect with our known size to avoid problems with X.
+
+And, to avoid problems with clever optimizations and float vs. double
+accuracy, we do the test using int:s.
 */
-  NSRect r2;
+  int x, y, w, h;
 
-  r = NSIntersectionRect(r, NSMakeRect(0, 0,
-    window->xframe.size.width, window->xframe.size.height));
-  if (NSIsEmptyRect(r))
+  x = floor(rect.origin.x);
+  y = floor(rect.origin.y);
+  w = ceil(rect.size.width + rect.origin.x - x);
+  h = ceil(rect.size.height + rect.origin.y - y);
+
+  if (x < 0)
+    {
+      w += x;
+      x = 0;
+    }
+  if (y < 0)
+    {
+      h += y;
+      y = 0;
+    }
+  if (x + w > window->xframe.size.width)
+    {
+      w = window->xframe.size.width - x;
+    }
+  if (y + h > window->xframe.size.height)
+    {
+      h = window->xframe.size.height - y;
+    }
+
+  if (w <= 0 || h <= 0)
     return;
 
-  r2.origin.x=floor(r.origin.x);
-  r2.origin.y=floor(r.origin.y);
-  r2.size.width=ceil(r.size.width+r.origin.x-r2.origin.x);
-  r2.size.height=ceil(r.size.height+r.origin.y-r2.origin.y);
-
-  r=r2;
-
   if (use_shm)
     {
 
@@ -407,7 +535,8 @@ static int warn = 0;
 	  else
 	    {
 //		NSLog(@"  updating");
-	      p = XCreatePixmapFromBitmapData(display, window->ident, buf, sx, sy, 1, 0, 1);
+	      p = XCreatePixmapFromBitmapData(display, window->ident,
+					      (char *)buf, sx, sy, 1, 0, 1);
 	      free(old_shape);
 	      old_shape = buf;
 	      old_shape_size = dsize;
@@ -422,21 +551,38 @@ static int warn = 0;
 	  if (!pending_put)
 	    {
 	      pending_put = 1;
-	      pending_rect = r;
+	      pending_rect.x = x;
+	      pending_rect.y = y;
+	      pending_rect.w = w;
+	      pending_rect.h = h;
 	    }
 	  else
 	    {
-	      pending_rect = NSUnionRect(pending_rect, r);
+	      if (x < pending_rect.x)
+		{
+		  pending_rect.w += pending_rect.x - x;
+		  pending_rect.x = x;
+		}
+	      if (x + w > pending_rect.x + pending_rect.w)
+		{
+		  pending_rect.w = x + w - pending_rect.x;
+		}
+	      if (y < pending_rect.y)
+		{
+		  pending_rect.h += pending_rect.y - y;
+		  pending_rect.y = y;
+		}
+	      if (y + h > pending_rect.y + pending_rect.h)
+		{
+		  pending_rect.h = y + h - pending_rect.y;
+		}
 	    }
 	}
       else
 	{
 	  pending_put = 0;
 	  if (!XShmPutImage(display, drawable, gc, ximage,
-			    r.origin.x, r.origin.y,
-			    r.origin.x, r.origin.y,
-			    r.size.width, r.size.height,
-			    1))
+			    x, y, x, y, w, h, 1))
 	    {
 	      NSLog(@"XShmPutImage failed?");
 	    }
@@ -460,9 +606,7 @@ static int warn = 0;
     }
   else if (ximage)
     XPutImage(display, drawable, gc, ximage,
-	      r.origin.x, r.origin.y,
-	      r.origin.x, r.origin.y,
-	      r.size.width, r.size.height);
+	      x, y, x, y, w, h);
 }
 
 -(void) needsAlpha
@@ -541,9 +685,6 @@ static int warn = 0;
 {
   NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
   use_shape_hack = [ud boolForKey: @"XWindowBuffer-shape-hack"];
-
-  if ([ud objectForKey: @"XWindowBufferUseXShm"])
-    use_xshm = [ud boolForKey: @"XWindowBufferUseXShm"];
 }
 
 
@@ -558,7 +699,6 @@ static int warn = 0;
 	  return;
 	}
     }
-  NSLog(@"Warning: gotShmCompletion: couldn't find XWindowBuffer for drawable");
 }
 
 @end
