@@ -33,6 +33,7 @@
 
 #include <signal.h>
 #include <unistd.h>
+#include <ctype.h>
 
 #ifdef __MINGW__
 #include	"process.h"
@@ -42,7 +43,7 @@
 #define	NSIG	32
 #endif
 
-static int	is_daemon = 0;		/* Currently running as daemon.	 */
+static BOOL	is_daemon = NO;		/* Currently running as daemon.	 */
 static char	ebuf[2048];
 
 #ifdef HAVE_SYSLOG
@@ -111,7 +112,6 @@ gpbs_log (int prio)
 @end
 static Class	xPbClass;
 
-int debug = 0;
 int verbose = 0;
 
 #define MAXHIST 100
@@ -1123,12 +1123,16 @@ ihandler(int sig)
 }
 
 static void
-init(int argc, char** argv)
+init(int argc, char** argv, char **env)
 {
-  NSUserDefaults *defs;
-  NSArray	*args = [[NSProcessInfo processInfo] arguments];
-  unsigned	count, c;
+  NSUserDefaults	*defs;
+  NSProcessInfo		*pInfo;
+  NSMutableArray	*args;
+  unsigned		count;
+  BOOL			shouldFork = YES;
 
+  pInfo = [NSProcessInfo processInfo];
+  args = AUTORELEASE([[pInfo arguments] mutableCopy]);
   for (count = 1; count < [args count]; count++)
     {
       NSString	*a = [args objectAtIndex: count];
@@ -1142,9 +1146,14 @@ init(int argc, char** argv)
 	  printf("--verbose\tMore verbose debug output\n");
 	  exit(EXIT_SUCCESS);
 	}
+      else if ([a isEqualToString: @"--daemon"] == YES)
+	{
+	  is_daemon = YES;
+	  shouldFork = NO;
+	}
       else if ([a isEqualToString: @"--no-fork"] == YES)
 	{
-	  debug++;
+	  shouldFork = NO;
 	}
       else if ([a isEqualToString: @"--verbose"] == YES)
 	{
@@ -1181,97 +1190,41 @@ init(int argc, char** argv)
 #endif 
   signal(SIGTERM, ihandler);
 
-  if (debug == 0)
+  if (shouldFork == YES)
     {
-      /*
-       *  Now fork off child process to run in background.
-       */
-#ifdef __MINGW__
-    {
-      char	**a = malloc((argc+2) * sizeof(char*));
+      NSFileHandle	*null;
+      NSTask		*t;
 
-      memcpy(a, argv, argc*sizeof(char*));
-      a[argc] = "--no-fork";
-      a[argc+1] = 0;
-      if (_spawnv(_P_NOWAIT, argv[0], a) == -1)
+      t = [NSTask new];
+      NS_DURING
 	{
-	  fprintf(stderr, "gpbs - spawn failed - bye.\n");
-	  exit(EXIT_FAILURE);
+	  [args removeObjectAtIndex: 0];
+	  [args addObject: @"--daemon"];
+	  [t setLaunchPath: [[NSBundle mainBundle] executablePath]];
+	  [t setArguments: args];
+	  [t setEnvironment: [pInfo environment]];
+	  null = [NSFileHandle fileHandleWithNullDevice];
+	  [t setStandardInput: null];
+	  [t setStandardOutput: null];
+	  [t setStandardError: null];
+	  [t launch];
+	  DESTROY(t);
 	}
-      exit(EXIT_SUCCESS);
-    }
-#else
-      is_daemon = 1;
-      switch (fork())
+      NS_HANDLER
 	{
-	  case -1:
-	    NSLog(@"gpbs - fork failed - bye.\n");
-	    exit(1);
-
-	  case 0:
-	    /*
-	     *	Try to run in background.
-	     */
-#ifdef	NeXT
-	    setpgrp(0, getpid());
-#else
-	    setsid();
-#endif
-	    break;
-
-	  default:
-	    if (verbose)
-	      {
-		NSLog(@"Process backgrounded (running as daemon)\r\n");
-	      }
-	    exit(EXIT_SUCCESS);
+	  gpbs_log(LOG_CRIT);
+	  DESTROY(t);
 	}
-#endif 
+      NS_ENDHANDLER
+      exit(EXIT_FAILURE);
     }
 
-#ifndef __MINGW__
   /*
-   *	Ensure we don't have any open file descriptors which may refer
-   *	to sockets bound to ports we may try to use.
-   *
-   *	Use '/dev/null' for stdin and stdout.
+   * Make gpbs logging go to syslog unless overridden by user.
    */
-  for (c = 0; c < FD_SETSIZE; c++)
-    {
-      if (is_daemon || (c != 2))
-	{
-	  (void)close(c);
-	}
-    }
-  if (open("/dev/null", O_RDONLY) != 0)
-    {
-      sprintf(ebuf, "failed to open stdin from /dev/null (%s)\n",
-	strerror(errno));
-      gpbs_log(LOG_CRIT);
-      exit(EXIT_FAILURE);
-    }
-  if (open("/dev/null", O_WRONLY) != 1)
-    {
-      sprintf(ebuf, "failed to open stdout from /dev/null (%s)\n",
-	strerror(errno));
-      gpbs_log(LOG_CRIT);
-      exit(EXIT_FAILURE);
-    }
-  if (is_daemon && open("/dev/null", O_WRONLY) != 2)
-    {
-      sprintf(ebuf, "failed to open stderr from /dev/null (%s)\n",
-	strerror(errno));
-      gpbs_log(LOG_CRIT);
-      exit(EXIT_FAILURE);
-    }
-#endif
-
-    /*
-     * Make gpbs logging go to syslog unless overridden by user.
-     */
-    defs = [NSUserDefaults standardUserDefaults];
-    [defs registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
-      @"YES", @"GSLogSyslog", nil]];
+  defs = [NSUserDefaults standardUserDefaults];
+  [defs registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
+    @"YES", @"GSLogSyslog", nil]];
 }
 
 
@@ -1285,7 +1238,7 @@ main(int argc, char** argv, char **env)
   [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
 #endif
 
-  init(argc, argv);
+  init(argc, argv, env);
 
   // [NSObject enableDoubleReleaseCheck: YES];
 
@@ -1358,6 +1311,16 @@ main(int argc, char** argv, char **env)
     {
       NSLog(@"GNU pasteboard server startup.\n");
     }
+
+  if ([[NSUserDefaults standardUserDefaults]
+	  stringForKey: @"GSStartupNotification"])
+    {
+      [[NSDistributedNotificationCenter defaultCenter]
+	postNotificationName: [[NSUserDefaults standardUserDefaults]
+				  stringForKey: @"GSStartupNotification"]
+		      object: nil];
+    }
+
   [[NSRunLoop currentRunLoop] run];
   RELEASE(server);
   RELEASE(pool);
